@@ -55,6 +55,8 @@ where
 {
     listener: EventListener<UserEvent>,
     subs: Vec<Subscription<ComponentId, UserEvent>>,
+    /// If true, subs won't be processed. (Default: False)
+    sub_lock: bool,
     view: View<ComponentId, Msg, UserEvent>,
 }
 
@@ -72,6 +74,7 @@ where
         Self {
             listener: listener_cfg.start(),
             subs: Vec::new(),
+            sub_lock: false,
             view: View::default(),
         }
     }
@@ -113,7 +116,9 @@ where
             .flatten()
             .collect();
         // Forward to subscriptions and extend vector
-        messages.extend(self.forward_to_subscriptions(events));
+        if !self.sub_lock {
+            messages.extend(self.forward_to_subscriptions(events));
+        }
         Ok(messages)
     }
 
@@ -128,7 +133,7 @@ where
         &mut self,
         id: K,
         component: WrappedComponent<Msg, UserEvent>,
-        subs: Vec<Sub<UserEvent>>,
+        subs: Vec<Sub<K, UserEvent>>,
     ) -> ApplicationResult<()> {
         // Mount
         self.view.mount(id.clone(), component)?;
@@ -162,7 +167,7 @@ where
         &mut self,
         id: K,
         component: WrappedComponent<Msg, UserEvent>,
-        subs: Vec<Sub<UserEvent>>,
+        subs: Vec<Sub<K, UserEvent>>,
     ) -> ApplicationResult<()> {
         let had_focus = self.view.has_focus(&id);
         let _ = self.umount(&id);
@@ -254,7 +259,7 @@ where
     ///
     /// Subscribe component to a certain event.
     /// Returns Error if the component doesn't exist or if the component is already subscribed to this event
-    pub fn subscribe(&mut self, id: &K, sub: Sub<UserEvent>) -> ApplicationResult<()> {
+    pub fn subscribe(&mut self, id: &K, sub: Sub<K, UserEvent>) -> ApplicationResult<()> {
         if !self.view.mounted(id) {
             return Err(ViewError::ComponentNotFound.into());
         }
@@ -281,6 +286,21 @@ where
         Ok(())
     }
 
+    /// ### lock_subs
+    ///
+    /// Lock subscriptions. As long as the subscriptions are locked, events won't be propagated to
+    /// subscriptions.
+    pub fn lock_subs(&mut self) {
+        self.sub_lock = true;
+    }
+
+    /// ### unlock_subs
+    ///
+    /// Unlock subscriptions. Application will now resume propagating events to subscriptions.
+    pub fn unlock_subs(&mut self) {
+        self.sub_lock = false;
+    }
+
     // -- private
 
     /// ### unsubscribe_component
@@ -375,8 +395,9 @@ where
                 }
                 if !sub.forward(
                     ev,
-                    |q| self.view.component(sub.target()).unwrap().query(q),
-                    || self.view.component(sub.target()).unwrap().state(),
+                    |id, q| self.view.query(id, q).ok().flatten(),
+                    |id| self.view.state(id).ok(),
+                    |id| self.view.mounted(id),
                 ) {
                     continue;
                 }
@@ -447,6 +468,7 @@ mod test {
             Application::init(listener_config());
         assert!(application.subs.is_empty());
         assert_eq!(application.view.mounted(&MockComponentId::InputFoo), false);
+        assert_eq!(application.sub_lock, false);
     }
 
     #[test]
@@ -550,11 +572,19 @@ mod test {
                     Sub::new(SubEventClause::Tick, SubClause::Always),
                     Sub::new(
                         SubEventClause::Tick,
-                        SubClause::HasAttrValue(Attribute::InputLength, AttrValue::Length(8))
+                        SubClause::HasAttrValue(
+                            MockComponentId::InputFoo,
+                            Attribute::InputLength,
+                            AttrValue::Length(8)
+                        )
                     ), // NOTE: This event will be ignored
                     Sub::new(
                         SubEventClause::User(MockEvent::Bar),
-                        SubClause::HasAttrValue(Attribute::Focus, AttrValue::Flag(true))
+                        SubClause::HasAttrValue(
+                            MockComponentId::InputFoo,
+                            Attribute::Focus,
+                            AttrValue::Flag(true)
+                        )
                     )
                 ]
             )
@@ -566,7 +596,11 @@ mod test {
                 &MockComponentId::InputFoo,
                 Sub::new(
                     SubEventClause::User(MockEvent::Foo),
-                    SubClause::HasAttrValue(Attribute::Focus, AttrValue::Flag(false))
+                    SubClause::HasAttrValue(
+                        MockComponentId::InputFoo,
+                        Attribute::Focus,
+                        AttrValue::Flag(false)
+                    )
                 )
             )
             .is_ok());
@@ -577,7 +611,11 @@ mod test {
                 &MockComponentId::InputFoo,
                 Sub::new(
                     SubEventClause::User(MockEvent::Foo),
-                    SubClause::HasAttrValue(Attribute::Focus, AttrValue::Flag(false))
+                    SubClause::HasAttrValue(
+                        MockComponentId::InputFoo,
+                        Attribute::Focus,
+                        AttrValue::Flag(false)
+                    )
                 )
             )
             .is_err());
@@ -587,7 +625,11 @@ mod test {
                 &MockComponentId::InputBar,
                 Sub::new(
                     SubEventClause::User(MockEvent::Foo),
-                    SubClause::HasAttrValue(Attribute::Focus, AttrValue::Flag(false))
+                    SubClause::HasAttrValue(
+                        MockComponentId::InputBar,
+                        Attribute::Focus,
+                        AttrValue::Flag(false)
+                    )
                 )
             )
             .is_err());
@@ -619,7 +661,11 @@ mod test {
                     Sub::new(SubEventClause::Tick, SubClause::Always),
                     Sub::new(
                         SubEventClause::User(MockEvent::Bar),
-                        SubClause::HasAttrValue(Attribute::Focus, AttrValue::Flag(true))
+                        SubClause::HasAttrValue(
+                            MockComponentId::InputFoo,
+                            Attribute::Focus,
+                            AttrValue::Flag(true)
+                        )
                     )
                 ]
             )
@@ -660,7 +706,11 @@ mod test {
                     Sub::new(
                         // NOTE: won't be thrown, since requires focus
                         SubEventClause::Keyboard(KeyEvent::from(Key::Enter)),
-                        SubClause::HasAttrValue(Attribute::Focus, AttrValue::Flag(true))
+                        SubClause::HasAttrValue(
+                            MockComponentId::InputBar,
+                            Attribute::Focus,
+                            AttrValue::Flag(true)
+                        )
                     )
                 ]
             )
@@ -700,13 +750,276 @@ mod test {
             &[MockMsg::BarSubmit(String::from(""))]
         );
         // Let's try TryFor strategy
-        assert!(
+        let events = application
+            .tick(PollStrategy::TryFor(Duration::from_millis(300)))
+            .ok()
+            .unwrap();
+        assert!(events.len() >= 2);
+    }
+
+    #[test]
+    fn should_not_propagate_event_when_subs_are_locked() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![
+                    Sub::new(SubEventClause::Tick, SubClause::Always),
+                    Sub::new(
+                        // NOTE: won't be thrown, since requires focus
+                        SubEventClause::Keyboard(KeyEvent::from(Key::Enter)),
+                        SubClause::HasAttrValue(
+                            MockComponentId::InputBar,
+                            Attribute::Focus,
+                            AttrValue::Flag(true)
+                        )
+                    )
+                ]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        // lock subs
+        application.lock_subs();
+        assert_eq!(application.sub_lock, true);
+        assert_eq!(
             application
-                .tick(PollStrategy::TryFor(Duration::from_millis(300)))
+                .tick(PollStrategy::UpTo(5))
                 .ok()
                 .unwrap()
-                .len()
-                >= 3
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from(""))]
+        );
+        // unlock subs
+        application.unlock_subs();
+        assert_eq!(application.sub_lock, false);
+    }
+
+    #[test]
+    fn should_not_propagate_events_if_has_attr_cond_is_not_satisfied() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![Sub::new(
+                    // NOTE: won't be thrown, since requires focus
+                    SubEventClause::Tick,
+                    SubClause::HasAttrValue(
+                        MockComponentId::InputBar,
+                        Attribute::Focus,
+                        AttrValue::Flag(true)
+                    )
+                )]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        assert_eq!(
+            application
+                .tick(PollStrategy::UpTo(5))
+                .ok()
+                .unwrap()
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from(""))]
+        );
+    }
+
+    #[test]
+    fn should_propagate_events_if_has_attr_cond_is_satisfied() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![Sub::new(
+                    SubEventClause::Tick,
+                    SubClause::HasAttrValue(
+                        MockComponentId::InputFoo,
+                        Attribute::Focus,
+                        AttrValue::Flag(true)
+                    )
+                )]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        assert_eq!(
+            application
+                .tick(PollStrategy::UpTo(5))
+                .ok()
+                .unwrap()
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from("")), MockMsg::BarTick]
+        );
+    }
+
+    #[test]
+    fn should_not_propagate_events_if_has_state_cond_is_not_satisfied() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![Sub::new(
+                    SubEventClause::Tick,
+                    SubClause::HasState(MockComponentId::InputFoo, State::None)
+                )]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        assert_eq!(
+            application
+                .tick(PollStrategy::UpTo(5))
+                .ok()
+                .unwrap()
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from(""))]
+        );
+    }
+
+    #[test]
+    fn should_propagate_events_if_has_state_cond_is_satisfied() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![Sub::new(
+                    SubEventClause::Tick,
+                    SubClause::HasState(
+                        MockComponentId::InputFoo,
+                        State::One(StateValue::String(String::new()))
+                    )
+                )]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        // No event should be generated
+        assert_eq!(
+            application
+                .tick(PollStrategy::UpTo(5))
+                .ok()
+                .unwrap()
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from("")), MockMsg::BarTick]
+        );
+    }
+
+    #[test]
+    fn should_not_propagate_events_if_is_mounted_cond_is_not_satisfied() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![Sub::new(
+                    SubEventClause::Tick,
+                    SubClause::IsMounted(MockComponentId::InputOmar)
+                )]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        assert_eq!(
+            application
+                .tick(PollStrategy::UpTo(5))
+                .ok()
+                .unwrap()
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from(""))]
+        );
+    }
+
+    #[test]
+    fn should_propagate_events_if_is_mounted_cond_is_not_satisfied() {
+        let mut application: Application<MockComponentId, MockMsg, MockEvent> =
+            Application::init(listener_config_with_tick(Duration::from_secs(60)));
+        // Mount foo and bar
+        assert!(application
+            .mount(
+                MockComponentId::InputFoo,
+                Box::new(MockFooInput::default()),
+                vec![]
+            )
+            .is_ok());
+        assert!(application
+            .mount(
+                MockComponentId::InputBar,
+                Box::new(MockBarInput::default()),
+                vec![Sub::new(
+                    SubEventClause::Tick,
+                    SubClause::IsMounted(MockComponentId::InputFoo)
+                )]
+            )
+            .is_ok());
+        // Active FOO
+        assert!(application.active(&MockComponentId::InputFoo).is_ok());
+        // No event should be generated
+        assert_eq!(
+            application
+                .tick(PollStrategy::UpTo(5))
+                .ok()
+                .unwrap()
+                .as_slice(),
+            &[MockMsg::FooSubmit(String::from("")), MockMsg::BarTick]
         );
     }
 
